package asd.graphe;

/**
 * Une classe pour les graphes orients reprsents  l'aide de listes
 * d'adjacence. Sauf mention contraire, toutes les mthodes sont en O(1).
 * (voir le <a href="../../../html/GrapheOriente.html">source</a>).
 * @author  2002 Marc Gatano
 * @version 2.0 (02/12/02)
 */
public class GrapheOriente implements Graphe {

    private int ordre;       // le nombre de sommets du graphe
    private int arcs;        // le nombre d'arcs du graphe
    private Adjacents[] adj; // les listes des sommets adjacents

    /**
     * Construit un graphe orient d'ordre n sans arc.
     * Complexit : O(n).
     */
    public GrapheOriente(int n) {
        ordre = n;
        arcs = 0;
        adj = new Adjacents[ordre];

        for ( int i = 0; i < adj.length; i++ ) {
            adj[i] = new Adjacents();
        }
    }

    /**
     * Retourne le nombre de sommets du Graphe.
     */
    public int ordre() {
        return ordre;
    }

    /**
     * Retourne le nombre d'arcs du graphe.
     */
    public int arcs() {
        return arcs;
    }

    /**
     * Ajoute l'arc (i,j) au graphe.
     * Prcondition : le sommet j n'est pas adjacent au sommet i.
     */
    public void connecter(int i, int j) {
        adj[i].ajouter(j);
        arcs++;
    }

    /**
     * Supprime l'arc (i,j) du graphe.
     * Prcondition : l'arc (i,j) existe.
     * Complexit : O(n) o n est l'ordre du graphe.
     */
    public void deconnecter(int i, int j) {
        adj[i].supprimer(j);
        arcs--;
    }

    /**
     * Retourne l'ensemble des sommets adjacents du sommet i.
     */
    public Adjacents adjacents(int i) {
        return (Adjacents) adj[i].clone();
    }
}
